<?xml version="1.1" encoding="utf-8"?>

<!-- 
Celle là fait la traduction du musicXML plat en braille non mis en page 
-->

<!DOCTYPE xsl:stylesheet SYSTEM "mmlents/windob.dtd">

<xsl:stylesheet version="2.0"
		xmlns:xs='http://www.w3.org/2001/XMLSchema'
		xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
		xmlns:fn='http://www.w3.org/2005/xpath-functions'
		xmlns:functx='http://www.functx.com' 
		xmlns:nat='http://natbraille.free.fr/xsl' >
  <xsl:output method="xml" indent="yes" />
  
  <xsl:include href="functions/functx-1.1alpha.xsl" /> 
  
  <xsl:variable name="dbg" select="fn:false()" />

  
  <!-- 
       Score-partwise 
  -->

  <xsl:template match="flat-score-partwise">
    <xsl:value-of select="./work" />
    <xsl:copy>
      <xsl:apply-templates select="part" />
    </xsl:copy>
  </xsl:template>

  <!-- 
       Part 
  -->
  <xsl:template match="part">    
    <xsl:element name = "part" >
      <xsl:value-of select="'&pt3456;&pt1;'"/>  <!-- not always  ... -->
      <xsl:call-template name="trAduc">
	<xsl:with-param name="in_num_child" select="1" />
	<xsl:with-param name="in_max_num_child" select="count(name|attributes|note|measure_bar)" /> 
	<xsl:with-param name="in_attributes" >
	  <xsl:call-template name="getAttributeDefautValues" />
	</xsl:with-param>
	<xsl:with-param name="in_accidentals_set" select="nat:get_armure_alters(0)"/>
	<xsl:with-param name="in_comparation_pitch" >
	</xsl:with-param>
      </xsl:call-template>
      <xsl:value-of select="'&pt126;&pt13;'"/>  <!-- not always  ... -->
    </xsl:element> 

  </xsl:template>

  <!-- 
       One recursive part iteration
  --> 
  <xsl:template name="trAduc">
    
    <xsl:param name="in_attributes" />
    <xsl:param name="in_num_child"  />
    <xsl:param name="in_max_num_child" />
    <xsl:param name="in_accidentals_set" />
    <xsl:param name="in_comparation_pitch" />

    <xsl:variable name="c_is_name" select="fn:not(fn:compare(local-name(*[$in_num_child]),'name'))" />
    <xsl:variable name="c_is_attribute_key" select="*[$in_num_child]/key/fifths" />
    <xsl:variable name="c_is_note" select="*[$in_num_child]/pitch/step" />
    <xsl:variable name="c_is_rest" select="*[$in_num_child]/rest" />
    <xsl:variable name="c_is_measure_bar" select="fn:not(fn:compare(local-name(*[$in_num_child]),'measure_bar'))" />
    <xsl:variable name="c_is_attribute_time" select="*[$in_num_child]/time/beats" />

    <!-- get this element attributes (this one or last one) -->
    <xsl:variable name="c_attributes">
      <xsl:copy-of select="if ($c_is_attribute_key) then (*[$in_num_child]) else ($in_attributes/attributes)" />
    </xsl:variable>

    <xsl:variable name="c_armure"> 
      <xsl:copy-of select="nat:get_armure_alters($c_attributes/attributes/key/fifths)" />
    </xsl:variable>   

    <!-- is the (potential) alteration accidental ? -->
    <xsl:variable name="alter_in_acci" 
		  select="if ($c_is_note)
			  then (nat:pitch_is_in(*[$in_num_child]/pitch,$in_accidentals_set))
			  else (fn:true())" />
    
    <!-- display measure_bar if it's a measure_bar -->
    <xsl:if test="$c_is_measure_bar" >
      <xsl:value-of select="nat:dbg('bar:')" />
      <xsl:value-of select="'&pt;'" />
    </xsl:if>

    <!-- display the time signature there's a time signature -->
    <xsl:if test="$c_is_attribute_time" >
      <xsl:value-of select="nat:dbg('time:')"/>
      <xsl:value-of select="nat:brl_timesig(*[$in_num_child]/time)" />
    </xsl:if>

    <!-- display the key signature if there's a key signature-->
    <xsl:if test="$c_is_attribute_key" >
      <xsl:value-of select="nat:dbg('key:')"/>
      <xsl:value-of select="nat:brl_keysig(*[$in_num_child]/key/fifths)" />
    </xsl:if>
    
    <!-- display rest if it's a note-->
    <xsl:if test="$c_is_rest" >
      <xsl:value-of select="nat:brl_rest_type(*[$in_num_child]/type,count(*[$in_num_child]/dot))"/>
    </xsl:if>

    <!-- display note if it's a note-->
    <xsl:if test="$c_is_note" >

      <!-- octave -->
      
      <xsl:variable name="octave_mark"
		    select="if ($in_comparation_pitch)  
			    then (nat:brl_octave_mark($in_comparation_pitch,*[$in_num_child]/pitch))
			    else (nat:brl_octave_mark(*[$in_num_child]/pitch))" />
      

      <xsl:if test="fn:compare('',$octave_mark)">
	<xsl:value-of select="nat:dbg('oct:')"/>
	<xsl:value-of select="$octave_mark"/>
      </xsl:if>

      <!-- accidental -->
      <xsl:if test="not($alter_in_acci)">
	<xsl:value-of select="nat:dbg('acc:')"/>
	<xsl:variable name="uti" select="*[$in_num_child]/pitch/alter" />   
	<xsl:value-of select="if ($uti &gt; 0) then ('&pt146;') else (if ($uti &lt; 0) then ('&pt126;') else('&pt16;'))" />
      </xsl:if>

      <!-- pitch & length -->
      <xsl:value-of select="nat:dbg('not:')"/>
      <xsl:value-of select="nat:brl_pitch_type(*[$in_num_child]/pitch/step,
			    *[$in_num_child]/type,
			    count(*[$in_num_child]/dot))" />
      
    </xsl:if>
    
    <xsl:variable name="out_accidentals_set"> 
      <xsl:choose>
	<xsl:when test="$c_is_note">
	  <xsl:copy-of select="nat:pitchset_set_pitch($in_accidentals_set,*[$in_num_child]/pitch)" />
	</xsl:when>
	<xsl:otherwise>
	  <xsl:copy-of select="$in_accidentals_set" />
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- recurse  -->
    <xsl:if test="($in_num_child &lt; $in_max_num_child)">     
      <xsl:call-template name="trAduc">
	<xsl:with-param name="in_attributes" select="$c_attributes" />
	<xsl:with-param name="in_num_child" select="($in_num_child+1)" />
	<xsl:with-param name="in_max_num_child"  select="$in_max_num_child" />
	<!-- pass an empty accidentals set if element is a measure_bar -->
	<xsl:with-param name="in_accidentals_set"  
			select="if ($c_is_measure_bar or $c_is_attribute_key) 
				then ($c_armure) 
				else ($out_accidentals_set)" />
	<xsl:with-param name="in_comparation_pitch" 
			select="if ($c_is_note) 
				then (*[$in_num_child]/pitch)
				else ($in_comparation_pitch)" />
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- 
       Default <Attributes> node
  --> 
  <xsl:template name="getAttributeDefautValues">
    <xsl:element name="attributes">
      <xsl:element name="time">
	<xsl:element name="beats" >
	  <xsl:number value="4" />
	</xsl:element>
	<xsl:element name="beat-type">
	  <xsl:number value="4" />
	</xsl:element>
      </xsl:element>
      <xsl:element name="key">
	<xsl:element name="fifths" >
	  <xsl:number value="0" />
	</xsl:element>
	<xsl:element name="mode">
	  <xsl:text>major</xsl:text>
	</xsl:element>
      </xsl:element>
    </xsl:element>
  </xsl:template>


  <!-- 
       nat:get_armure_alters
       - prend +1 pour # , +2 pour ##, -3 pour bbb
       - renvoie une liste des notes avec leur altération de l'armure
       <note>  <step>C</step>  <alter>0</step>   </note>
       <note>  <step>D</step>  <alter>0</step>   </note>
       <note>  <step>E</step>  <alter>0</step>   </note>
       <note>  <step>F</step>  <alter>1</step>   </note>
       ...
  -->

  <xsl:function name="nat:get_armure_alters" as="node()*" >
    <xsl:param name="fifths"  as="xs:integer"/>
    <xsl:variable name='alterations'>
      <xsl:choose>
	<xsl:when test="$fifths &gt; 0">
	  <xsl:value-of select="fn:substring('FCGDAEB',1,$fifths)" />
	</xsl:when>
	<xsl:when test="$fifths &lt; 0">
	  <xsl:value-of select="fn:substring('BEADGCF',1,fn:abs($fifths))" />
	</xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="sequence_notes" as="xs:string*" select="('C','D','E','F','G','A','B')" />
    <xsl:for-each select="$sequence_notes">
      <xsl:element name="pitch">
	<xsl:element name="step">
	  <xsl:value-of select="." />
	</xsl:element>
	<xsl:element name="alter">
	  <xsl:choose>
	    <xsl:when test="contains($alterations,.)">
	      <xsl:value-of select="if ($fifths &gt; 0) then (+1) else (-1)" />
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="0" />
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:element>
      </xsl:element>
    </xsl:for-each>
  </xsl:function>


  <!--
      nat:brl_keysig
      - prend un <fifths> 
      - renvoie l'armure en braille     &pt146; dieze ; &pt126; bémol ; &pt16; becarre
  -->
  <xsl:function name="nat:brl_keysig" as="xs:string" >
    <xsl:param name="fifths" as="xs:integer"/>

    <xsl:variable name="abs" select="fn:abs($fifths)" />
    <xsl:variable name="al_sign" select="if ($abs &lt; 0)  then ('&pt126;')  else (if ($abs &gt; 0) then ('&pt146;') else ('') )" />
    
    <xsl:value-of select="if ($abs = 1) then  ($al_sign) else (
			  if ($abs = 2) then (fn:concat($al_sign,$al_sign)) else (
			  if ($abs = 3) then (fn:concat($al_sign,$al_sign,$al_sign)) else (
			  if ($abs &gt; 3) then (fn:concat($abs,$al_sign)) else ())))
			  "/>

  </xsl:function>

  <!--
      nat:pitch_is_in
      - prend un <pitch> et un set de <pitch>
      - renvoie vrai si l'<alter> du <pitch> est la même que celle
      de l'<alter> du <pitch> correspondant dans le set de <pitch>s
  -->

  <xsl:function name="nat:pitch_is_in" as="xs:boolean" >
    
    <xsl:param name="pitch" as="node()"/>
    <xsl:param name="gamme" as="node()"/>
    
    <xsl:variable name="c_step" select="$pitch/step" />
    <xsl:variable name="c_alter" select="if ($pitch/alter) then ($pitch/alter) else (0)"/>   
    
    <xsl:variable name="alter_of_it">
      <xsl:for-each select="$gamme/pitch" >
	<xsl:if test="contains(./step,$c_step)" >
	  <xsl:value-of select="./alter" />
	</xsl:if>
      </xsl:for-each>
    </xsl:variable>
    
    <xsl:value-of select="if ($alter_of_it = $c_alter) then (fn:true()) else (fn:false())" />

  </xsl:function>

  <!--
      nat:pitch_set_add
      - prend set de <pitch> (gamme) et un <pitch>
      - renvoie un nouveau set de <pitch> id. à celui d'entrée
      avec l'alter du pitch passé en paramètre
  -->

  <xsl:function name="nat:pitchset_set_pitch" as="node()*" >
    
    <xsl:param name="gamme" as="node()"/>
    <xsl:param name="pitch" as="node()"/>
    
    <xsl:variable name="c_step" select="$pitch/step" />
    <xsl:variable name="c_alter" select="if ($pitch/alter) then ($pitch/alter) else (0)"/>   
    
    <xsl:for-each select="$gamme/pitch" >
      <xsl:element name="pitch">
	<xsl:element name="step">
	  <xsl:value-of select="./step" />
	</xsl:element>
	<xsl:element name="alter">
	  <xsl:choose>
	    <xsl:when test="./step = $c_step" >
	      <xsl:value-of select="$c_alter" />
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="./alter" />
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:element>
      </xsl:element>
    </xsl:for-each>
  </xsl:function>
  
  <!--
      nat:brl_octave_mark
      - takes a <pitch>  
      - returns octave mark 
  -->
  <xsl:function name="nat:brl_octave_mark" as="xs:string">
    <xsl:param name="pitch_t" as="node()"/>

    <xsl:variable name="brl_octave_marks">
      <h>&pt4;</h>
      <h>&pt45;</h>
      <h>&pt456;</h>
      <h>&pt5;</h>
      <h>&pt46;</h>
      <h>&pt56;</h>
      <h>&pt6;</h>
    </xsl:variable>
    <xsl:value-of select="$brl_octave_marks/h[position()=$pitch_t/octave]"/>
  </xsl:function>

  <!--
      nat:brl_octave_mark
      - takes an old <pitch> and a new <pitch>
      - returns octave mark if needed
  -->
  <xsl:function name="nat:brl_octave_mark" as="xs:string">

    <xsl:param name="pitch_f" as="node()"/>
    <xsl:param name="pitch_t" as="node()"/>

    <xsl:variable name="abs_interval_class" select="1 + abs(nat:pitches_to_interval_class($pitch_f,$pitch_t))" />

    <!--
	if the distance between the two notes 
	1. is an interval of <= 3rd     => no octave mark
	2. is an interval of >= 6th     => octave mark
	3. is an interval of 4th or 5th
	3a. and two notes in the same octave => no octave mark
	3b. and two notes in different octave => octave mark
	
	surprise ! il faut aussi une marque d'octave lorsque l'on est passé à la ligne !
    -->

    <xsl:variable name="om">
      <xsl:choose>
	<xsl:when test="$abs_interval_class &gt; 5">
	  <xsl:value-of select="$pitch_t/octave" />
	</xsl:when>
	<xsl:when test="$abs_interval_class &gt; 3">
	  <xsl:value-of select="if ($pitch_f/octave = $pitch_t/octave) then ('') else ($pitch_t/octave)" />
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="''" />
	</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
   

    <xsl:choose>
      <xsl:when test="$om != ''">
	<xsl:value-of select="nat:brl_octave_mark($pitch_t)"/>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="''" />
      </xsl:otherwise>
    </xsl:choose>

  </xsl:function>


  <!-- 
       nat:pitches_to_interval_class
       - takes an old <pitch> and a new <pitch>
       - return the interval class as <integer>
  -->
  <xsl:function name="nat:pitches_to_interval_class" as="xs:integer">
    <xsl:param name="pitch_f" as="node()"/>
    <xsl:param name="pitch_t" as="node()"/>

    <xsl:variable name="note_equiv">
      <h k="C">0</h>
      <h k="D">1</h>
      <h k="E">2</h>
      <h k="F">3</h>
      <h k="G">4</h>
      <h k="A">5</h>
      <h k="B">6</h>
    </xsl:variable>

    <xsl:variable name="abs_deg_f" select="functx:id-untyped($note_equiv,$pitch_f/step) + (7 * $pitch_f/octave)" />
    <xsl:variable name="abs_deg_t" select="functx:id-untyped($note_equiv,$pitch_t/step) + (7 * $pitch_t/octave)" />
    <xsl:value-of select="($abs_deg_t - $abs_deg_f)" />

  </xsl:function>

  <!-- 
       nat:brl_pitch_type
       - takes a <step> , a <type> and the number of dots
       - return the corresponding brl symbol
  -->
  <xsl:function name="nat:brl_pitch_type" as="xs:string">

    <xsl:param name="step" as="xs:string"/>  <!-- C,D,E,... -->
    <xsl:param name="type" as="xs:string"/>  <!-- 'whole' ou 'half' ou 'quarter' ou 'eight' .... -->
    <xsl:param name="dot"  as="xs:integer"/>  <!-- 'dot'  .... -->

    <xsl:variable name="type_equiv">
      <h k="whole" >whole</h>
      <h k="half">half</h>
      <h k="quarter">quarter</h>
      <h k="eighth">eighth</h>
      <h k="16th" >whole</h>
      <h k="32th">half</h>
      <h k="64th">quarter</h>
      <h k="128th">eighth</h>
    </xsl:variable>

    <xsl:variable name="pt">
      <h k="Cwhole">&pt1345;</h>
      <h k="Dwhole">&pt135;</h>
      <h k="Ewhole">&pt12346;</h>
      <h k="Fwhole">&pt123456;</h>
      <h k="Gwhole">&pt12356;</h>
      <h k="Awhole">&pt2346;</h>
      <h k="Bwhole">&pt23456;</h>
      <h k="Chalf">&pt1345;</h>
      <h k="Dhalf">&pt135;</h>
      <h k="Ehalf">&pt1234;</h>
      <h k="Fhalf">&pt12345;</h>
      <h k="Ghalf">&pt1235;</h>
      <h k="Ahalf">&pt234;</h>
      <h k="Bhalf">&pt2345;</h>
      <h k="Cquarter">&pt1456;</h>
      <h k="Dquarter">&pt156;</h>
      <h k="Equarter">&pt1246;</h>
      <h k="Fquarter">&pt12456;</h>
      <h k="Gquarter">&pt1256;</h>
      <h k="Aquarter">&pt246;</h>
      <h k="Bquarter">&pt2456;</h>
      <h k="Ceighth">&pt145;</h>
      <h k="Deighth">&pt15;</h>
      <h k="Eeighth">&pt124;</h>
      <h k="Feighth">&pt1245;</h>
      <h k="Geighth">&pt125;</h>
      <h k="Aeighth">&pt24;</h>
      <h k="Beigtht">&pt245;</h>
    </xsl:variable>

    <xsl:variable name="r_type" select="functx:id-untyped($type_equiv,$type)" />
    <xsl:value-of>
      <xsl:value-of select="functx:id-untyped($pt,fn:concat($step,$r_type))" />
      <xsl:value-of select="functx:repeat-string('&pt3;', $dot)" />
    </xsl:value-of>

  </xsl:function>
  
  <!-- 
       nat:brl_rest_type
       correponds to nat:brl_pitch_type for rests
  -->
  <xsl:function name="nat:brl_rest_type" as="xs:string">
    <xsl:param name="type" as="xs:string"/>  <!-- 'whole' ou 'half' ou 'quarter' ou 'eight' .... -->
    <xsl:param name="dot"  as="xs:integer"/>  <!-- 'dot'  .... -->

    <xsl:variable name="type_equiv">
      <h k="whole" >whole</h>
      <h k="half">half</h>
      <h k="quarter">quarter</h>
      <h k="eighth">eighth</h>
      <h k="16th" >whole</h>
      <h k="32th">half</h>
      <h k="64th">quarter</h>
      <h k="128th">eighth</h>
    </xsl:variable>

    <xsl:variable name="pt">
      <h k="whole">&pt134;</h>
      <h k="half">&pt136;</h>
      <h k="quarter">&pt1236;</h>
      <h k="eighth">&pt1346;</h>
    </xsl:variable>

    <xsl:variable name="r_type" select="functx:id-untyped($type_equiv,$type)" />

    <xsl:value-of>
      <xsl:value-of select="functx:id-untyped($pt,$r_type)" />
      <xsl:value-of select="functx:repeat-string('&pt3;', $dot)" />
    </xsl:value-of>

  </xsl:function>
  
  
  <!-- 
       nat:brl_timesig
       - gets a <time> (of <attribute>)
       - returns the brl time signature
  -->
  <xsl:function name="nat:brl_timesig" as="xs:string"> <!-- La mesure se place en début de morceau entre deux espaces. -->
    <xsl:param name="time" as="node()" />
    
    <xsl:variable name="brl_beats">
      <h k="0">&pt245;</h>
      <h k="1">&pt1;</h>
      <h k="2">&pt12;</h>
      <h k="3">&pt14;</h>
      <h k="4">&pt145;</h>
      <h k="5">&pt15;</h>
      <h k="6">&pt124;</h>
      <h k="7">&pt1245;</h>
      <h k="8">&pt125;</h>
      <h k="9">&pt24;</h>
    </xsl:variable>

    <xsl:variable name="brl_beats_type">
      <h k="0">&pt356;</h>
      <h k="1">&pt2;</h>
      <h k="2">&pt23;</h>
      <h k="3">&pt25;</h>
      <h k="4">&pt256;</h>
      <h k="5">&pt26;</h>
      <h k="6">&pt235;</h>
      <h k="7">&pt2356;</h>
      <h k="8">&pt236;</h>
      <h k="9">&pt35;</h>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="($time/beats=4) and ($time/beat-type=4)" >
	<xsl:text>&pt46;&pt14;</xsl:text> <!-- C -->
      </xsl:when>
      <xsl:when test="($time/beats=2) and ($time/beat-type=2)" >
	<xsl:text>&pt456;&pt14;</xsl:text> <!-- C barre-->
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of >
	  <xsl:for-each select="functx:chars($time/beats)" >
	    <xsl:value-of select="functx:id-untyped($brl_beats,.)" />
	  </xsl:for-each>
	  <xsl:for-each select="functx:chars($time/beat-type)" >
	    <xsl:value-of select="functx:id-untyped($brl_beats_type,.)" />
	  </xsl:for-each>
	</xsl:value-of>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:function>

  <xsl:function name="nat:dbg" as="xs:string?" >
    <xsl:param name="txt" as="xs:string" />
    <xsl:if test="$dbg">
      <xsl:value-of select=" $txt " />
    </xsl:if>
  </xsl:function>
  
</xsl:stylesheet>
